@@ -22,6 +22,7 @@ import (
 	"sort"
 	"strconv"
 	"testing"
+	"time"
 
 	"github.com/google/go-cmp/cmp"
 	"go.uber.org/zap/zapcore"
@@ -76,17 +77,6 @@ var (
 	obsConfig        metrics.ObservabilityConfig
 	traceConfig      tracingconfig.Config
 	defaults, _      = apicfg.NewDefaultsConfigFromMap(nil)
-	revCfg           = config.Config{
-		Config: &apicfg.Config{
-			Autoscaler: &asConfig,
-			Defaults:   defaults,
-		},
-		Deployment:    &deploymentConfig,
-		Logging:       &logConfig,
-		Network:       &network.Config{},
-		Observability: &obsConfig,
-		Tracing:       &traceConfig,
-	}
 )
 
 const testProbeJSONTemplate = `{"tcpSocket":{"port":%d,"host":"127.0.0.1"}}`
@@ -99,6 +89,7 @@ func TestMakeQueueContainer(t *testing.T) {
 		nc   network.Config
 		oc   metrics.ObservabilityConfig
 		dc   deployment.Config
+		fc   apicfg.Features
 		want corev1.Container
 	}{{
 		name: "autoscaler single",
@@ -127,6 +118,7 @@ func TestMakeQueueContainer(t *testing.T) {
 		want: queueContainer(func(c *corev1.Container) {
 			c.Image = "alpine"
 			c.Ports = append(queueNonServingPorts, queueHTTP2Port)
+			c.ReadinessProbe.Handler.HTTPGet.Port.IntVal = queueHTTP2Port.ContainerPort
 			c.Env = env(map[string]string{
 				"USER_PORT":          "1955",
 				"QUEUE_SERVING_PORT": "8013",
@@ -309,6 +301,68 @@ func TestMakeQueueContainer(t *testing.T) {
 				"METRICS_COLLECTOR_ADDRESS":       "otel:55678",
 			})
 		}),
+	}, {
+		name: "HTTP2 autodetection enabled",
+		rev: revision("bar", "foo",
+			withContainers(containers)),
+		fc: apicfg.Features{
+			AutoDetectHTTP2: apicfg.Enabled,
+		},
+		want: queueContainer(func(c *corev1.Container) {
+			c.Env = env(map[string]string{
+				"ENABLE_HTTP2_AUTO_DETECTION": "true",
+			})
+		}),
+	}, {
+		name: "set concurrency state endpoint",
+		rev: revision("bar", "foo",
+			withContainers(containers)),
+		dc: deployment.Config{
+			ConcurrencyStateEndpoint: "freeze-proxy",
+		},
+		want: queueContainer(func(c *corev1.Container) {
+			c.Env = env(map[string]string{
+				"CONCURRENCY_STATE_ENDPOINT": "freeze-proxy",
+			})
+		}),
+	}, {
+		name: "HTTP2 autodetection disabled",
+		rev: revision("bar", "foo",
+			withContainers(containers)),
+		fc: apicfg.Features{
+			AutoDetectHTTP2: apicfg.Disabled,
+		},
+		dc: deployment.Config{
+			ProgressDeadline: 0 * time.Second,
+		},
+		want: queueContainer(func(c *corev1.Container) {
+			c.Env = env(map[string]string{
+				"ENABLE_HTTP2_AUTO_DETECTION": "false",
+			})
+		}),
+	}, {
+		name: "set startup probe timeouts based on ProgressDeadline",
+		rev: revision("bar", "foo",
+			withContainers(containers)),
+		dc: deployment.Config{
+			ProgressDeadline: 1984 * time.Second,
+		},
+		want: queueContainer(func(c *corev1.Container) {
+			c.Env = env(map[string]string{})
+			c.StartupProbe = &corev1.Probe{
+				Handler: corev1.Handler{
+					Exec: &corev1.ExecAction{
+						Command: []string{"/ko-app/queue", "-probe-timeout", "33m4s"},
+					},
+				},
+				// The startup probe timeout seconds is set to the progress deadline to
+				// match the timeout on initial deployment.
+				TimeoutSeconds:   1984,
+				PeriodSeconds:    1,
+				SuccessThreshold: 1,
+				FailureThreshold: 1,
+			}
+		}),
 	}}
 
 	for _, test := range tests {
@@ -326,6 +380,9 @@ func TestMakeQueueContainer(t *testing.T) {
 				Logging:       &test.lc,
 				Observability: &test.oc,
 				Deployment:    &test.dc,
+				Config: &apicfg.Config{
+					Features: &test.fc,
+				},
 			}
 			got, err := makeQueueContainer(test.rev, cfg)
 			if err != nil {
@@ -459,7 +516,7 @@ func TestMakeQueueContainerWithPercentageAnnotation(t *testing.T) {
 
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
-			cfg := (&revCfg).DeepCopy()
+			cfg := revConfig()
 			cfg.Deployment = &test.dc
 			got, err := makeQueueContainer(test.rev, cfg)
 			if err != nil {
@@ -521,10 +578,15 @@ func TestProbeGenerationHTTPDefaults(t *testing.T) {
 		c.Env = env(map[string]string{
 			"SERVING_READINESS_PROBE": string(wantProbeJSON),
 		})
+		c.StartupProbe = nil // User overrode periodSeconds so no exec probe needed.
 		c.ReadinessProbe = &corev1.Probe{
 			Handler: corev1.Handler{
-				Exec: &corev1.ExecAction{
-					Command: []string{"/ko-app/queue", "-probe-period", "10s"},
+				HTTPGet: &corev1.HTTPGetAction{
+					Port: intstr.FromInt(int(queueHTTPPort.ContainerPort)),
+					HTTPHeaders: []corev1.HTTPHeader{{
+						Name:  network.ProbeHeaderName,
+						Value: queue.Name,
+					}},
 				},
 			},
 			PeriodSeconds:  1,
@@ -532,7 +594,7 @@ func TestProbeGenerationHTTPDefaults(t *testing.T) {
 		}
 	})
 
-	got, err := makeQueueContainer(rev, &revCfg)
+	got, err := makeQueueContainer(rev, revConfig())
 	if err != nil {
 		t.Fatal("makeQueueContainer returned error")
 	}
@@ -593,17 +655,23 @@ func TestProbeGenerationHTTP(t *testing.T) {
 			"USER_PORT":               strconv.Itoa(userPort),
 			"SERVING_READINESS_PROBE": string(wantProbeJSON),
 		})
+		c.StartupProbe = nil // User overrode periodSeconds so no execprobe needed.
 		c.ReadinessProbe = &corev1.Probe{
 			Handler: corev1.Handler{
-				Exec: &corev1.ExecAction{
-					Command: []string{"/ko-app/queue", "-probe-period", "10s"},
-				}},
+				HTTPGet: &corev1.HTTPGetAction{
+					Port: intstr.FromInt(int(queueHTTPPort.ContainerPort)),
+					HTTPHeaders: []corev1.HTTPHeader{{
+						Name:  network.ProbeHeaderName,
+						Value: queue.Name,
+					}},
+				},
+			},
 			PeriodSeconds:  2,
 			TimeoutSeconds: 10,
 		}
 	})
 
-	got, err := makeQueueContainer(rev, &revCfg)
+	got, err := makeQueueContainer(rev, revConfig())
 	if err != nil {
 		t.Fatal("makeQueueContainer returned error")
 	}
@@ -620,6 +688,7 @@ func TestTCPProbeGeneration(t *testing.T) {
 		rev       v1.RevisionSpec
 		want      corev1.Container
 		wantProbe *corev1.Probe
+		dc        deployment.Config
 	}{{
 		name: "knative tcp probe",
 		wantProbe: &corev1.Probe{
@@ -650,15 +719,38 @@ func TestTCPProbeGeneration(t *testing.T) {
 				}},
 			},
 		},
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
 		want: queueContainer(func(c *corev1.Container) {
-			c.ReadinessProbe = &corev1.Probe{
+			c.StartupProbe = &corev1.Probe{
 				Handler: corev1.Handler{
 					Exec: &corev1.ExecAction{
-						Command: []string{"/ko-app/queue", "-probe-period", "0"},
+						Command: []string{"/ko-app/queue", "-probe-timeout", "1h34m38s"},
 					},
 				},
-				PeriodSeconds:  10,
-				TimeoutSeconds: 10,
+				// StartupProbe overrides the user's parameters because the
+				// actual probing happens inside the execprobe.
+				// The timeout seconds is set to the progress deadline to match the
+				// timeout on initial deployment.
+				TimeoutSeconds:   5678,
+				PeriodSeconds:    1,
+				SuccessThreshold: 1,
+				FailureThreshold: 1,
+			}
+			c.ReadinessProbe = &corev1.Probe{
+				Handler: corev1.Handler{
+					HTTPGet: &corev1.HTTPGetAction{
+						Port: intstr.FromInt(int(queueHTTPPort.ContainerPort)),
+						HTTPHeaders: []corev1.HTTPHeader{{
+							Name:  network.ProbeHeaderName,
+							Value: queue.Name,
+						}},
+					},
+				},
+				PeriodSeconds:    1,
+				TimeoutSeconds:   0,
+				SuccessThreshold: 3,
 			}
 			c.Env = env(map[string]string{"USER_PORT": strconv.Itoa(userPort)})
 		}),
@@ -689,14 +781,20 @@ func TestTCPProbeGeneration(t *testing.T) {
 			TimeoutSeconds: 1,
 		},
 		want: queueContainer(func(c *corev1.Container) {
+			c.StartupProbe = nil // User overrode periodSeconds, no exec probe needed.
 			c.ReadinessProbe = &corev1.Probe{
 				Handler: corev1.Handler{
-					Exec: &corev1.ExecAction{
-						Command: []string{"/ko-app/queue", "-probe-period", "1s"},
+					HTTPGet: &corev1.HTTPGetAction{
+						Port: intstr.FromInt(int(queueHTTPPort.ContainerPort)),
+						HTTPHeaders: []corev1.HTTPHeader{{
+							Name:  network.ProbeHeaderName,
+							Value: queue.Name,
+						}},
 					},
 				},
-				PeriodSeconds:  1,
-				TimeoutSeconds: 1,
+				PeriodSeconds: 1,
+				// Inherit Kubernetes default here rather than overriding as we need to do for exec probe.
+				TimeoutSeconds: 0,
 			}
 			c.Env = env(map[string]string{})
 		}),
@@ -737,10 +835,15 @@ func TestTCPProbeGeneration(t *testing.T) {
 			},
 		},
 		want: queueContainer(func(c *corev1.Container) {
+			c.StartupProbe = nil // User overrode periodSeconds, no exec probe needed.
 			c.ReadinessProbe = &corev1.Probe{
 				Handler: corev1.Handler{
-					Exec: &corev1.ExecAction{
-						Command: []string{"/ko-app/queue", "-probe-period", "15s"},
+					HTTPGet: &corev1.HTTPGetAction{
+						Port: intstr.FromInt(int(queueHTTPPort.ContainerPort)),
+						HTTPHeaders: []corev1.HTTPHeader{{
+							Name:  network.ProbeHeaderName,
+							Value: queue.Name,
+						}},
 					},
 				},
 				PeriodSeconds:       2,
@@ -767,7 +870,10 @@ func TestTCPProbeGeneration(t *testing.T) {
 				Value: string(wantProbeJSON),
 			})
 
-			got, err := makeQueueContainer(testRev, &revCfg)
+			config := revConfig()
+			config.Deployment = &test.dc
+
+			got, err := makeQueueContainer(testRev, config)
 			if err != nil {
 				t.Fatal("makeQueueContainer returned error")
 			}
@@ -781,29 +887,30 @@ func TestTCPProbeGeneration(t *testing.T) {
 }
 
 var defaultEnv = map[string]string{
-	"CONTAINER_CONCURRENCY":                 "0",
-	"ENABLE_PROFILING":                      "false",
-	"METRICS_DOMAIN":                        metrics.Domain(),
-	"METRICS_COLLECTOR_ADDRESS":             "",
-	"QUEUE_SERVING_PORT":                    "8012",
-	"REVISION_TIMEOUT_SECONDS":              "45",
-	"SERVING_CONFIGURATION":                 "",
-	"SERVING_ENABLE_PROBE_REQUEST_LOG":      "false",
-	"SERVING_ENABLE_REQUEST_LOG":            "false",
-	"SERVING_LOGGING_CONFIG":                "",
-	"SERVING_LOGGING_LEVEL":                 "",
-	"SERVING_NAMESPACE":                     "foo",
-	"SERVING_REQUEST_LOG_TEMPLATE":          "",
-	"SERVING_REQUEST_METRICS_BACKEND":       "",
-	"SERVING_REVISION":                      "bar",
-	"SERVING_SERVICE":                       "",
-	"SYSTEM_NAMESPACE":                      system.Namespace(),
-	"TRACING_CONFIG_BACKEND":                "",
-	"TRACING_CONFIG_DEBUG":                  "false",
-	"TRACING_CONFIG_SAMPLE_RATE":            "0",
-	"TRACING_CONFIG_STACKDRIVER_PROJECT_ID": "",
-	"TRACING_CONFIG_ZIPKIN_ENDPOINT":        "",
-	"USER_PORT":                             strconv.Itoa(v1.DefaultUserPort),
+	"CONCURRENCY_STATE_ENDPOINT":       "",
+	"CONTAINER_CONCURRENCY":            "0",
+	"ENABLE_HTTP2_AUTO_DETECTION":      "false",
+	"ENABLE_PROFILING":                 "false",
+	"METRICS_DOMAIN":                   metrics.Domain(),
+	"METRICS_COLLECTOR_ADDRESS":        "",
+	"QUEUE_SERVING_PORT":               "8012",
+	"REVISION_TIMEOUT_SECONDS":         "45",
+	"SERVING_CONFIGURATION":            "",
+	"SERVING_ENABLE_PROBE_REQUEST_LOG": "false",
+	"SERVING_ENABLE_REQUEST_LOG":       "false",
+	"SERVING_LOGGING_CONFIG":           "",
+	"SERVING_LOGGING_LEVEL":            "",
+	"SERVING_NAMESPACE":                "foo",
+	"SERVING_REQUEST_LOG_TEMPLATE":     "",
+	"SERVING_REQUEST_METRICS_BACKEND":  "",
+	"SERVING_REVISION":                 "bar",
+	"SERVING_SERVICE":                  "",
+	"SYSTEM_NAMESPACE":                 system.Namespace(),
+	"TRACING_CONFIG_BACKEND":           "",
+	"TRACING_CONFIG_DEBUG":             "false",
+	"TRACING_CONFIG_SAMPLE_RATE":       "0",
+	"TRACING_CONFIG_ZIPKIN_ENDPOINT":   "",
+	"USER_PORT":                        strconv.Itoa(v1.DefaultUserPort),
 }
 
 func probeJSON(container *corev1.Container) string {
@@ -853,3 +960,18 @@ func sortEnv(envs []corev1.EnvVar) {
 func resourcePtr(q resource.Quantity) *resource.Quantity {
 	return &q
 }
+
+func revConfig() *config.Config {
+	return &config.Config{
+		Config: &apicfg.Config{
+			Autoscaler: &asConfig,
+			Defaults:   defaults,
+			Features:   &apicfg.Features{},
+		},
+		Deployment:    &deploymentConfig,
+		Logging:       &logConfig,
+		Network:       &network.Config{},
+		Observability: &obsConfig,
+		Tracing:       &traceConfig,
+	}
+}
